// Dynamic link library implementation of NeuroSolutions LineKohonen component 

#include "NSDLL.h" 

/*************************************************************/
/* Macros to access the PE layers and weights in matrix form */

#define in(i,j)		input[j+i*inCols]
#define out(i,j)	output[j+i*outCols]
#define W(i,j)		weights[j+i*inCount]

/******************************/
/* Unsupervised weight update */

__declspec(dllexport) void performKohonen(
	DLLData	*instance,		// Pointer to instance data (may be NULL)
	NSFloat	*input,			// Pointer to input layer 
	int 	inRows,			// Number of rows of PEs in the input layer
	int 	inCols,			// Number of columns of PEs in the input layer
	NSFloat	*output, 		// Pointer to output layer
	int		outRows, 		// Number of rows of PEs in the output layer
	int		outCols, 		// Number of columns of PEs in the output layer
	NSFloat	*weights,		// Pointer to fully connected weight matrix 
	NSFloat	step, 			// Learning rate
	int		winningRow,		// Index of winning row 
	int		winningCol,		// Index of winning column 
	int		size 			// Size of the neighborhood
	)
{	
	int	i,j,
		inCount = inRows*inCols,
		outCount = outRows * outCols,
		winner = winningCol + winningRow*outCols,
		start = winner - size,
		stop = winner + size;

	if (start < 0)
		start = 0;
	if (stop >= outCount)
		stop = outCount-1;
	for (i=start; i<=stop; i++)
		for (j=0; j<inCount; j++)
			W(i,j) += step*(input[j] - W(i,j));
}

/******************************************/
/* Management of instance data (OPTIONAL) */
/*
__declspec(dllexport) DLLData *allocKohonen(
	DLLData	*oldInstance,	// Pointer to the last instance if reallocating
	int 	inRows,		// Number of rows of PEs in the input layer
	int 	inCols,		// Number of columns of PEs in the input layer
	int 	outRows,	// Number of rows of PEs in the output layer
	int 	outCols		// Number of columns of PEs in the output layer
	)
{
	DLLData *instance = allocDLLInstance(oldInstance);
	return instance;
}

__declspec(dllexport) void freeKohonen(DLLData *instance)
{
	freeDLLInstance(instance);
}
*/