// Dynamic link library implementation of NeuroSolutions BackTanhAxon component 

#include "NSDLL.h"

/********************************/
/* Backpropagation of component */

__declspec(dllexport) void performBackLinearAxon(
	DLLData *instance,	// Pointer to instance data (may be NULL)
	DLLData *dualInstance,	// Pointer to the forward axons instance data (may be NULL)
	NSFloat	*data, 		// Pointer to the layer of processing elements (PEs)
	int 	rows,		// Number of rows of PEs in the layer
	int 	cols,		// Number of columns of PEs in the layer
	NSFloat	*error, 	// Pointer to the sensitivity vector
	NSFloat	*gradient, 	// Pointer to the bias gradient vector
	NSFloat	beta		// Slope gain scalar, same for all PEs
	)
	
{
	int i, length=rows*cols;

	for (i=0; i<length; i++) {
		error[i] *= beta*(1.0f - data[i]*data[i] + 0.1f);
		if (gradient)
			gradient[i] += error[i];
	} 
}

/******************************************/
/* Management of instance data (OPTIONAL) */
/*
__declspec(dllexport) DLLData *allocBackLinearAxon(
	DLLData	*oldInstance,	// Pointer to the last instance if reallocating
	DLLData	*dualInstance,	// Pointer to forward axons instance data (may be NULL)
	int 	rows,		// Number of rows of PEs in the layer
	int 	cols		// Number of columns of PEs in the layer
	)
{
	DLLData *instance = allocDLLInstance(oldInstance);
	return instance;
}

__declspec(dllexport) void freeBackLinearAxon(DLLData *instance)
{
	freeDLLInstance(instance);
}
*/