// Dynamic link library implementation of NeuroSolutions GaussianAxon component 

#include "NSDLL.h"

/************************************************/
/* Macro to access the PE layer in matrix form */

#define data(i,j)		data[j+i*cols]

/***********************************/
/* Forward activation of component */

__declspec(dllexport) void performLinearAxon(
	DLLData *instance,	// Pointer to instance data (may be NULL)
	NSFloat	*data, 		// Pointer to the layer of processing elements (PEs)
	int 	rows,		// Number of rows of PEs in the layer
	int 	cols,		// Number of columns of PEs in the layer
	NSFloat	*bias,		// Pointer to the layer's bias vector, one for each PE
	NSFloat	beta		// Slope gain scalar, same for all PEs
	)
{
	int i, length=rows*cols;

	for (i=0; i<length; i++) {
		data[i] += bias[i];
		data[i] = (NSFloat)exp(-beta*data[i]*data[i]);
	}
}

/******************************************/
/* Management of instance data (OPTIONAL) */
/*
__declspec(dllexport) DLLData *allocLinearAxon(
	DLLData	*oldInstance,	// Pointer to the last instance if reallocating
	int 	rows,		// Number of rows of PEs in the layer
	int 	cols		// Number of columns of PEs in the layer
	)
{
	DLLData *instance = allocDLLInstance(oldInstance);
	return instance;
}

__declspec(dllexport) void freeLinearAxon(DLLData *instance)
{
	freeDLLInstance(instance);
}
*/